@@ -62,6 +62,7 @@ class DockerMethodRunner(MethodRunner):
             dataset=dataset,
             dataset_part=dataset_part,
             hyperparameters=hyperparameters,
+            method_runner_type="docker",
             goal=goal,
             stage=stage,
             method=method,
@@ -78,7 +79,7 @@ class DockerMethodRunner(MethodRunner):
         else:
             self.resultsFolder = os.path.join(resultsFolder, self.id)
 
-    def run(self):
+    def run(self, cpu_count=1, mem_limit="4g"):
         logging.info(f"Running {self.id} with params {self.hyperparameters}")
         folder_parameters = tempfile.TemporaryDirectory()
         path_options = os.path.join(folder_parameters.name, "options.yml")
@@ -125,8 +126,8 @@ class DockerMethodRunner(MethodRunner):
                         "mode": "ro",
                     }
                 },
-                mem_limit="20g",
-                cpu_count=7,
+                mem_limit=mem_limit,
+                cpu_count=cpu_count,
                 detach=True,
             )
 
@@ -181,8 +182,8 @@ class DockerMethodRunner(MethodRunner):
                 logging.error(
                     f"Runner {self.id} errored with status code {status['StatusCode']}!"
                 )
-                # for line in e.container.logs().decode().split("\n"):
-                #     logging.error(f"Container[{self.image}]: " + line)
+                for line in container.logs().decode().split("\n"):
+                    logging.error(f"Container[{self.image}]: " + line)
                 if status["StatusCode"] == 137:
                     logging.error("Process in container was killed.")
                     logging.error(
