@@ -13,7 +13,7 @@ import os
 from glob import glob
 import json
 from typing import Literal, TypedDict, Dict
-from ldimbenchmark.constants import LDIM_BENCHMARK_CACHE_DIR
+from ldimbenchmark.constants import CPU_COUNT, LDIM_BENCHMARK_CACHE_DIR
 import shutil
 import hashlib
 from pandas import DataFrame
@@ -25,7 +25,7 @@ import copy
 import yaml
 from yaml import CDumper
 from yaml.representer import SafeRepresenter
-import datetime
+from ldimbenchmark.utilities import dirhash
 
 
 # Fix for Timestamp parsing/dumping in yaml
@@ -37,7 +37,7 @@ def timestamp_representer(dumper, data):
     return SafeRepresenter.represent_datetime(dumper, data.to_pydatetime())
 
 
-TSDumper.add_representer(datetime.datetime, SafeRepresenter.represent_datetime)
+TSDumper.add_representer(datetime, SafeRepresenter.represent_datetime)
 TSDumper.add_representer(pd.Timestamp, timestamp_representer)
 
 
@@ -113,7 +113,7 @@ def loadDatasetsDirectly(
         data_dir_in_dataset_dir = os.path.join(dataset_path, data_dir)
         if not os.path.exists(data_dir_in_dataset_dir):
             raise FileNotFoundError(
-                f"Could not find the {data_dir} directory in the dataset directory"
+                f"Could not find the {data_dir} directory in the dataset directory ({dataset_path})"
             )
         datasets[data_dir] = {}
         for sensor_readings_file in glob(
@@ -125,7 +125,7 @@ def loadDatasetsDirectly(
                 index_col="Timestamp",
                 chunksize=100000,
             )
-            with ThreadPoolExecutor(max_workers=os.cpu_count() - 1) as executor:
+            with ThreadPoolExecutor(max_workers=CPU_COUNT) as executor:
                 frames = list(executor.map(parse_frame_dates, d))
 
             sensor_readings = pd.concat(frames)
@@ -170,24 +170,60 @@ class Dataset:
         """
         self.path = path
         self.__pickle_path = os.path.join(self.path, "dataset.pickle")
+        self.__validation_file_name = ".validation"
+        self.__validation_path = os.path.join(self.path, self.__validation_file_name)
         path_to_dataset_info = os.path.join(self.path, "dataset_info.yaml")
         # Read dataset_info.yaml
         if os.path.isfile(path_to_dataset_info):
             # file exists
             with open(path_to_dataset_info) as f:
                 self.info: DatasetInfo = yaml.safe_load(f)
-                self.info["dataset"]["training"]["start"] = pd.to_datetime(
+                training_start = pd.to_datetime(
                     self.info["dataset"]["training"]["start"], utc=True
                 )
-                self.info["dataset"]["training"]["end"] = pd.to_datetime(
+                training_end = pd.to_datetime(
                     self.info["dataset"]["training"]["end"], utc=True
                 )
-                self.info["dataset"]["evaluation"]["start"] = pd.to_datetime(
+                evaluation_start = pd.to_datetime(
                     self.info["dataset"]["evaluation"]["start"], utc=True
                 )
-                self.info["dataset"]["evaluation"]["end"] = pd.to_datetime(
+                evaluation_end = pd.to_datetime(
                     self.info["dataset"]["evaluation"]["end"], utc=True
                 )
+                if training_start >= training_end:
+                    raise Exception(
+                        "Training start time is after or the same as training end time! (dataset_info.yaml)"
+                    )
+                if evaluation_start >= evaluation_end:
+                    raise Exception(
+                        "Evaluation start time is after or the same as evaluation end time! (dataset_info.yaml)"
+                    )
+                if training_start > evaluation_start:
+                    raise Exception(
+                        "Training start time is after evaluation start time! (dataset_info.yaml)"
+                    )
+                if training_end > evaluation_end:
+                    raise Exception(
+                        "Training end time is after evaluation end time! (dataset_info.yaml)"
+                    )
+                if evaluation_start < training_end:
+                    raise Exception(
+                        "Evaluation start time is before training end time! (dataset_info.yaml)"
+                    )
+                if evaluation_end < training_end:
+                    raise Exception(
+                        "Evaluation end time is before training end time! (dataset_info.yaml)"
+                    )
+                if training_start == training_end:
+                    raise Exception(
+                        "Training start time is equal to training end time! (dataset_info.yaml)"
+                    )
+
+                self.info["dataset"]["training"]["start"] = training_start
+                self.info["dataset"]["training"]["end"] = training_end
+                self.info["dataset"]["evaluation"]["start"] = evaluation_start
+                self.info["dataset"]["evaluation"]["end"] = evaluation_end
+
         else:
             raise Exception(
                 f"No dataset_info.yaml file found! (not at: '{path_to_dataset_info}')"
@@ -206,22 +242,52 @@ class Dataset:
             logging.warning("No dmas.json file found. Skipping loading of DMAs.")
             self.dmas = None
 
-    def _update_id(self):
+    def _update_id(self, force=False):
         """
         Sets the id (hash) according to the information in "dataset_info.yaml"
         """
 
-        # if "derivations" in self.info:
         derivations_hash = (
             "-"
             + hashlib.md5(
                 json.dumps(self.info, sort_keys=True, default=str).encode("utf-8")
             ).hexdigest()
         )
-        # else:
-        #     derivations_hash = ""
         self.id = self.info["name"] + derivations_hash
 
+    def _generate_dataset_hash(self, folder: str):
+        new_hash = dirhash(folder, "md5", ignore_hidden=True)
+        with open(os.path.join(folder, self.__validation_file_name), "w") as f:
+            f.write(str(pd.to_datetime(datetime.now(), utc=True)) + "\n" + new_hash)
+
+    def is_valid(self) -> bool:
+        # Check the has of the whole dataset from time to time by writing the last timestamp
+        # to a file and retrieving it before running the expensive hash function
+        md5hash = None
+        try:
+            with open(self.__validation_path, "r") as f:
+                timestamp_file = f.readlines()
+                last_timestamp = pd.to_datetime(timestamp_file[0], utc=True)
+                md5hash = timestamp_file[1]
+        except (FileNotFoundError, IndexError):
+            logging.warn("Dataset had no hashes for validation.")
+            last_timestamp = None
+
+        if last_timestamp is None or last_timestamp < pd.to_datetime(
+            datetime.now(), utc=True
+        ) - pd.Timedelta("24 hour"):
+            logging.info("Checking dataset consistency...")
+            new_hash = dirhash(self.path, "md5", ignore_hidden=True)
+            if md5hash != None and md5hash != new_hash:
+                logging.info(
+                    "Dataset hash changed! Check the consistency of the dataset!"
+                )
+                return False
+            md5hash = new_hash
+            with open(self.__validation_path, "w") as f:
+                f.write(str(pd.to_datetime(datetime.now(), utc=True)) + "\n" + md5hash)
+        return True
+
     ######
     # Getters and Setters with direct lazy loading because they
     ######
@@ -356,7 +422,7 @@ class Dataset:
                 for sensor in getattr(self, sensor_type).keys()
             ]
             # logging.debug(filepaths)
-            with ThreadPoolExecutor(max_workers=os.cpu_count() - 1) as executor:
+            with ThreadPoolExecutor(max_workers=CPU_COUNT) as executor:
                 # submit all tasks and get future objects
                 futures = [
                     executor.submit(write_to_csv, sensor, path)
@@ -377,6 +443,7 @@ class Dataset:
             )
         if os.path.exists(self.__pickle_path):
             os.remove(self.__pickle_path)
+        self._generate_dataset_hash(folder)
 
 
 def getTimeSliceOfDataset(
