@@ -29,7 +29,7 @@ class MNF(LDIMMethodBase):
     def __init__(self):
         super().__init__(
             name="mnf",
-            version="1.0",
+            version="1.2",
             metadata=MethodMetadata(
                 data_needed=["flow"],
                 hyperparameters=[
@@ -81,6 +81,7 @@ class MNF(LDIMMethodBase):
             resample_frequency=self.hyperparameters["resample_frequency"],
         )
 
+        # If the data is too short, return an empty list
         if (
             simple_evaluation_data.flows.index[-1]
             - simple_evaluation_data.flows.index[0]
@@ -88,77 +89,137 @@ class MNF(LDIMMethodBase):
         ):
             return []
 
-        evaluation_start_date: Timestamp = simple_evaluation_data.flows.index[0]
+        day_counts = simple_evaluation_data.flows.groupby(
+            simple_evaluation_data.flows.index.date
+        ).size()
 
-        start_date: Timestamp = simple_evaluation_data.flows.index[0].replace(
+        if day_counts[day_counts.index[0]] != day_counts[day_counts.index[1]]:
+            first_full_day = day_counts.index[1]
+        else:
+            first_full_day = day_counts.index[0]
+
+        start_date = pd.to_datetime(first_full_day, utc=True).replace(
             hour=12, minute=0, second=0, microsecond=0, nanosecond=0
         )
-        # TODO: find better cut interval function
-        end_date: Timestamp = simple_evaluation_data.flows.index[-100].replace(
+
+        if day_counts[day_counts.index[-1]] != day_counts[day_counts.index[-2]]:
+            last_full_day = day_counts.index[-2]
+        else:
+            last_full_day = day_counts.index[-1]
+
+        end_date = pd.to_datetime(last_full_day, utc=True).replace(
             hour=12, minute=0, second=0, microsecond=0, nanosecond=0
         )
+
+        all_flows = simple_evaluation_data.flows.loc[
+            (simple_evaluation_data.flows.index >= start_date)
+            & (simple_evaluation_data.flows.index < end_date)
+        ]
+
         if self.simple_train_data:
+            # Use training data to set up the window, so we can start with the evaluation data
             previous_data = self.simple_train_data.flows
-            mask = (previous_data.index >= (start_date - window)) & (
-                previous_data.index < evaluation_start_date
+            previous_data_day_counts = previous_data.groupby(
+                previous_data.index.date
+            ).size()
+
+            if (
+                previous_data_day_counts[previous_data_day_counts.index[-1]]
+                != previous_data_day_counts[previous_data_day_counts.index[-2]]
+            ):
+                last_full_day = previous_data_day_counts.index[-2]
+            else:
+                last_full_day = previous_data_day_counts.index[-1]
+
+            previous_end_date = pd.to_datetime(last_full_day, utc=True).replace(
+                hour=12, minute=0, second=0, microsecond=0, nanosecond=0
+            )
+            if previous_data.index[-1] - previous_data.index[0] < 3 * window:
+                if (
+                    previous_data_day_counts[previous_data_day_counts.index[0]]
+                    != previous_data_day_counts[previous_data_day_counts.index[1]]
+                ):
+                    first_full_day = previous_data_day_counts.index[1]
+                else:
+                    first_full_day = previous_data_day_counts.index[0]
+
+                previous_start_date = pd.to_datetime(first_full_day, utc=True).replace(
+                    hour=12, minute=0, second=0, microsecond=0, nanosecond=0
+                )
+            else:
+                previous_start_date = previous_end_date - window
+
+            mask = (previous_data.index >= previous_start_date) & (
+                previous_data.index < previous_end_date
             )
             previous_data = self.simple_train_data.flows.loc[mask]
 
-            all_flows = pd.concat([previous_data, simple_evaluation_data.flows], axis=0)
-        else:
-            start_date: Timestamp = simple_evaluation_data.flows.index[100].replace(
-                hour=12, minute=0, second=0, microsecond=0, nanosecond=0
-            )
-            all_flows = simple_evaluation_data.flows[
-                simple_evaluation_data.flows.index >= start_date
-            ]
-
-        all_flows = all_flows.loc[all_flows.index < end_date]
+            all_flows = pd.concat([previous_data, all_flows], axis=0)
 
         # TODO: For now lets say it starts at noon
         hour_24_end = start_date + timedelta(days=1)
 
-        # better: all_flows.groupby(all_flows.index.date).size()
+        # better:
         entries_per_day = (
             (all_flows.index > start_date) & (all_flows.index <= hour_24_end)
         ).sum()
 
         days = int(all_flows.shape[0] / entries_per_day)
 
-        flows_array = all_flows[all_flows.columns[0]].to_numpy()
+        results = []
+        # all_flows = pd.DataFrame(all_flows.sum(axis=1))
+        for sensor in all_flows.columns:
+            flows_array = all_flows[sensor].to_numpy()
 
-        reshaped = np.reshape(flows_array, (days, entries_per_day))
+            # Error here
+            # array of size 58175 into shape (201, 288)
+            reshaped = np.reshape(flows_array, (days, entries_per_day))
 
-        min_flows = reshaped.min(axis=1)
+            min_flows = reshaped.min(axis=1)
 
-        labels = np.zeros(days)
-        # start the search for leaks at time window + first day
-        current_analysis_day = window.days + 1
-        while current_analysis_day < days:
-            min_window = min(
-                min_flows[current_analysis_day - window.days : current_analysis_day]
-            )
-            residual = min_flows[current_analysis_day] - min_window
-
-            # If residual is greater than gamma times the minimum window flow
-            if residual > min_window * gamma:
-                labels[current_analysis_day] = 1
-
-            current_analysis_day += 1
-
-        full_labels = np.repeat(labels, entries_per_day)
-
-        # Pattern search for change in labels
-        searchval = [0, 1]
-        leaks = all_flows.index[
-            np.concatenate(
-                (
-                    (full_labels[:-1] == searchval[0])
-                    & (full_labels[1:] == searchval[1]),
-                    [False],
+            labels = np.zeros(days)
+            # start the search for leaks at time window + first day
+            current_analysis_day = window.days + 1
+            while current_analysis_day < days:
+                min_window = min(
+                    min_flows[current_analysis_day - window.days : current_analysis_day]
                 )
-            )
-        ]
+                residual = min_flows[current_analysis_day] - min_window
+
+                # If residual is greater than gamma times the minimum window flow
+                if residual > min_window * gamma:
+                    labels[current_analysis_day] = 1
+
+                current_analysis_day += 1
+
+            full_labels = np.repeat(labels, entries_per_day)
+
+            # Pattern search for change in labels
+            searchval = [0, 1]
+            leaks = all_flows.index[
+                np.concatenate(
+                    (
+                        (full_labels[:-1] == searchval[0])
+                        & (full_labels[1:] == searchval[1]),
+                        [False],
+                    )
+                )
+            ]
+
+            for leak_start in leaks:
+                results.append(
+                    BenchmarkLeakageResult(
+                        leak_pipe_id=sensor,
+                        leak_time_start=leak_start,
+                        leak_time_end=leak_start,
+                        leak_time_peak=leak_start,
+                        leak_area=0.0,
+                        leak_diameter=0.0,
+                        leak_max_flow=0.0,
+                    )
+                )
+        return results
+
         # for i=0; i<days; i++:
         #     min_window = min(min_flows[i:i+window.days])
         #     if min_flows[current_analysis_day] - min_window > min_window * gamma:
@@ -218,20 +279,5 @@ class MNF(LDIMMethodBase):
         #     Labels_Sc = [datestr(timeStamps, 'yyyy-mm-dd HH:MM') repmat(', ',length(timeStamps),1) num2str(repmat(Labels_Sc_Final1, 1))];
         #     Labels_Sc = cellstr(Labels_Sc);
 
-        results = []
-        for leak_start in leaks:
-            results.append(
-                BenchmarkLeakageResult(
-                    leak_pipe_id=None,
-                    leak_time_start=leak_start,
-                    leak_time_end=leak_start,
-                    leak_time_peak=leak_start,
-                    leak_area=0.0,
-                    leak_diameter=0.0,
-                    leak_max_flow=0.0,
-                )
-            )
-        return results
-
     def detect_online(self, evaluation_data):
         pass
