@@ -3,7 +3,7 @@ from ldimbenchmark.classes import MethodMetadata, MethodMetadataDataNeeded
 from typing import List, Union
 
 
-class NullLeakagedDetectionMethod(LDIMMethodBase):
+class NullLeakageDetectionMethod(LDIMMethodBase):
     """
     Null Algorithm
     """
@@ -24,9 +24,11 @@ class NullLeakagedDetectionMethod(LDIMMethodBase):
                 hyperparameters=[],
             )
             # hyperparameters={"est_length": "3 days", "C_threshold": 3, "delta": 4},
-        )
+        )#
+    
+    
 
-    def train(self, train_data: BenchmarkData) -> None:
+    def prepare(self, train_data: BenchmarkData) -> None:
         return
 
     def detect_offline(
